#!/bin/bash

##
## This script can be used after a Java version upgrade, to update
##   Java settings for Talend applications and services.
## The Java path is stored in many places and without this script,
##   Talend applications will continue to run with the old Java version.
##
## Before running this script, check and set the following variables defined at the beginning of the script:
## - TALEND_HOME - Talend home folder. For example, "/opt/Talend-8.0.1".
## - PROD_VER    - Product version. For example, "8.0.1".
## - JAVA_HOME   - New Java home. This value will be set for Talend applications.
##
## Supported Talend versions: 7.3 and 8.0.
## Supported Java versions: 8, 11, and 17. Java 17 is only supported for Talend 8.0.1 R2023-10 or above.
##
## Recommended procedure for updating the Java settings for Talend applications and services:
## 1. Install the new Java version without removing the old one yet.
## 2. Stop all Talend services with systemctl stop <service>.
## 3. Update this script with the actual values for the TALEND_HOME, PROD_VER, and JAVA_HOME variables.
## 4. If needed, set the executable flag for this script by using the following command: chmod +x update_java.sh.
## 5. Run this script to update Java for all Talend applications.
## 6. Start all Talend applications either with systemctl start <service>, or by rebooting the machine.
## 7. (Optional) Remove the old Java version.
##
## Important: the following services don't support Java 11 nor Java 17 and will not be updated:
##     - Talend Data Quality Portal (obsolete in Talend 8 and later)
##     - Talend Nexus service       (obsolete in Talend 8 and later)
##     - Talend LogServer           (deprecated in Talend 8)
##
## Important: the Talend MDM service doesn't support Java 17 and will not be updated with that version.

TALEND_HOME=/opt/Talend-8.0.1
PROD_VER=8.0.1
JAVA_HOME=/usr/lib/jvm/java-11-openjdk-11.0.8.10-0.el7_8.x86_64

# This user is used by system services ('talenduser' by default)
SRV_USER=talenduser

# This script will modify system files, so it must be running with root privileges
if [ "`id -u`" != "0" ]; then
  echo "This script requires root privileges."
  echo "Re-run this script as root."
  exit 1
fi

# Check that we have 'sed' with in-place feature
if [ "`sed --help | grep -c -e '--in-place'`" = "0" ]; then
  echo "Error: sed version is too old."
  echo "This script requires 'sed' utility that supports in-place file update."
  echo "Upgrade your sed utility version and then re-run this script again."
  exit 1
fi

# Check environment
if [ ! -f ${TALEND_HOME}/uninstall ]; then
  echo "Error: variable TALEND_HOME is wrong: no Talend installation is detected."
  exit 1
fi

if [ ! -f ${JAVA_HOME}/bin/java ]; then
  echo "Error: variable JAVA_HOME is wrong: Java is not detected at that location."
  exit 1
fi

JAVA_VER=`${JAVA_HOME}/bin/java -version 2>&1 | grep version | sed 's/^[^"]*"//' | sed 's/".*//'`
JAVA_11=`echo $JAVA_VER | sed 's/^11\..*/OK/'`
JAVA_17=`echo $JAVA_VER | sed 's/^17\..*/OK/'`

# Check compatibility: Java 11 can be run only with Talend 7.3 or 8.0
if [ "$JAVA_11" = "OK" ]; then
  case "$PROD_VER" in
    7.3.* | 8.0.* )
      # all is fine
      ;;
    * )
      echo "Error: Java 11 is supported only for Talend 7.3 or 8.0."
      exit 1
      ;;
  esac
fi

errorJava17 () {
  echo "Error: target Java is Java 17, but Talend software version is older than 8.0.1 R2023-10."
  echo "       Setting up Java 17 for that version will make the system unusable."
  echo "Java upgrade is stopped."
  exit 1
}

checkJava17tomcats () {
  for app in tac iam mdm tds dq_dict
  do
    if [ -f $TALEND_HOME/$app/apache-tomcat/RELEASE-NOTES ]; then
      tomcatVersion=`cat $TALEND_HOME/$app/apache-tomcat/RELEASE-NOTES |grep "Apache Tomcat Version " | sed 's/\r$//g' | sed 's/.*Version //'`
      tomcatMajorVersion=`echo $tomcatVersion| sed 's/\..*//'`
      tomcatMinorVersion=`echo $tomcatVersion| sed 's/.*\.//'`
      if [ "$tomcatMajorVersion" -lt "9" -o "$tomcatMajorVersion" -eq "9" -a "$tomcatMinorVersion" -lt "82" ]; then
        errorJava17
      fi
    fi
  done
}

checkJava17dataprep () {
  if [ -d $TALEND_HOME/dataprep/lib ]; then
    # The script will take a version of dataprep-core jar file
    dpVer=`ls -1 $TALEND_HOME/dataprep/lib/dataprep-core-*.jar | grep -v actions | sed 's#.*/##' | sed 's/dataprep-core-//' | sed 's/\.jar//'`
    dpMajor=`echo $dpVer| sed 's/\..*//'`
    dpMinor=`echo $dpVer| sed 's/^[^\.]*\.//' | sed 's/\..*//'`
    if [ -z "$dpVer" -o "$dpMajor" -lt "5" -o "$dpMajor" -eq "5" -a "$dpMinor" -lt "14" ]; then
      errorJava17
    fi
  fi
}

checkJava17runtime () {
  if [ -f $TALEND_HOME/runtime/version.txt ]; then
    # Runtime version is checked indirectly with version of org/hibernate/hibernate-core dependency
    # For 8.0.1.R2023-10-RT it should be 5.6.7.Final (this script checks only 5.6 or above)
    rtVer=`ls -1 $TALEND_HOME/runtime/system/org/hibernate/hibernate-core`
    rtMajor=`echo $rtVer| sed 's/\..*//'`
    rtMinor=`echo $rtVer| sed 's/^[^\.]*\.//' | sed 's/\..*//'`
    if [ -z "$rtVer" -o "$rtMajor" -lt "5" -o "$rtMajor" -eq "5" -a "$rtMinor" -lt "6" ]; then
      errorJava17
    fi
  fi
}

# Check compatibility: Java 17 can be run only with Talend 8.0
if [ "$JAVA_17" = "OK" ]; then
  case "$PROD_VER" in
    8.0.* )
      checkJava17tomcats
      checkJava17dataprep
      checkJava17runtime
      ;;
    * )
      echo "Error: Java 17 is supported only for Talend 8.0."
      exit 1
      ;;
  esac
fi

if [ "`grep -c $SRV_USER /etc/passwd`" = "0" ]; then
  echo "Error: cannot find local user $SRV_USER."
  echo "This is the user for system services."
  echo "Check script variable SRV_USER and update it if needed."
  exit 1
fi

# Extra check that new Java can be executed by talenduser
# Check that java executable has "others" "executable" right ('????????x' file mask in 'ls -l' output)
# In addition, all containing folders must have "read" and "execute" rights for "others" ('??????r?x' mask)
cVal=`ls -l $JAVA_HOME/bin/java | awk '{ print substr($1,10,1) }'`
if [ "$cVal" != "x" ]; then
  echo "Error: Java executable $JAVA_HOME/bin/java does not have eXecute rights for 'others'."
  exit 1
fi

# Checking all folders in the path for 'rx' rights for 'other' users
dirJava="$JAVA_HOME/bin"
while [ "$dirJava" != "/" ]
do
  readRight=`ls -ld $dirJava | awk '{ print substr($1,8,1) }'`
  execRight=`ls -ld $dirJava | awk '{ print substr($1,10,1) }'`
  if [ "$readRight" != "r" ]; then
    echo "Error: folder $dirJava does not have 'Read' right for 'other' users."
    echo "Java from $JAVA_HOME cannot be used by system services."
    exit 1
  fi
  if [ "$execRight" != "x" ]; then
    echo "Error: folder $dirJava does not have 'eXecute' right for 'other' users."
    echo "Java from $JAVA_HOME cannot be used by system services."
    exit 1
  fi
  dirJava=`dirname $dirJava`
done

# Check that Talend services are not running
SRV_LIST=`(cd /etc/systemd/system; ls -1 talend-*-${PROD_VER}.service 2>/dev/null)`
if [ -z "$SRV_LIST" ]; then
  echo "Error: cannot detect any Talend $PROD_VER service."
  echo "Verify that the variable PROD_VER is set correctly."
  exit 1
fi

for cSrv in $SRV_LIST
do
  cStat=`systemctl is-active $cSrv`
  if [ "$?" = "0" ]; then
    echo "Error: not all Talend services are stopped."
    echo "Stop all Talend services and re-try."
    exit 1
  fi
done

echo
echo Updating Talend services to use Java $JAVA_VER from ${JAVA_HOME}...
echo

if [ "$JAVA_17" = "OK" ]; then
  if [ -f /etc/systemd/system/talend-mdm-${PROD_VER}.service ]; then
    echo "Warning: Talend MDM service will not be updated for Java 17 and will remain on the old Java version."
  fi
  if [ -f /etc/systemd/system/talend-elastic-${PROD_VER}.service ]; then
    echo "Warning: Talend LogServer service will not be updated for Java 17 and will remain on the old Java version."
  fi
fi

# First updating systemd service descriptor files with new JAVA_HOME
if [ "$JAVA_17" != "OK" ]; then
  # Update case for Java 8 / Java 11
  for cSrv in kafka zookeeper nexus cmdline elastic kibana logstash
  do
    if [ -f /etc/systemd/system/talend-$cSrv-${PROD_VER}.service ]; then
      if [ "$cSrv" != "nexus" -o "$JAVA_11" != "OK" ]; then
        echo "Updating systemd descriptor for Talend $cSrv service..."
        sed -i "s|Environment=JAVA_HOME=.*|Environment=JAVA_HOME=$JAVA_HOME|" /etc/systemd/system/talend-$cSrv-${PROD_VER}.service
      fi
    fi
  done
else
  # Update case for Java 17
  for cSrv in kafka zookeeper
  do
    if [ -f /etc/systemd/system/talend-$cSrv-${PROD_VER}.service ]; then
      echo "Updating systemd descriptor for Talend $cSrv service..."
      sed -i "s|Environment=JAVA_HOME=.*|Environment=JAVA_HOME=$JAVA_HOME|" /etc/systemd/system/talend-$cSrv-${PROD_VER}.service
    fi
  done
fi

# 2nd updating scripts in utils folder
if [ "$JAVA_17" != "OK" ]; then
  # Update case for Java 8 and Java 11
  scripts="dqdict iam mdm tac tds cmdline dataprep tcomp jobserver bpm kafka mongodb"
else
  # Update case for Java 17
  scripts="dqdict iam tac tds dataprep tcomp jobserver kafka mongodb"
fi

for cScript in $scripts
do
  if [ -f $TALEND_HOME/utils/srv_$cScript ]; then
    echo "Updating script utils/srv_$cScript ..."
    sed -i "s|JAVA_HOME=.*|JAVA_HOME=$JAVA_HOME|" $TALEND_HOME/utils/srv_$cScript
  fi
done

# Extra care for dataprep and tcomp services
for cScript in dataprep tcomp
do
  if [ -f $TALEND_HOME/utils/srv_$cScript ]; then
    sed -i "s|if \[ -n \"/.*\" \]; then|if \[ -n \"$JAVA_HOME\" \]; then|" $TALEND_HOME/utils/srv_$cScript
  fi
done

# Updating JAVA_BIN for DataPrep and TCOMP
for cFile in dataprep/start.sh dataprep/services/tcomp/start.sh
do
  if [ -f $TALEND_HOME/$cFile ]; then
    sed -i "s|JAVA_BIN=.*|JAVA_BIN=$JAVA_HOME/bin/java|" $TALEND_HOME/$cFile
  fi
done

# Updating Runtime service
if [ -f $TALEND_HOME/runtime/bin/talend-runtime-$PROD_VER-service ]; then
  echo "Updating Runtime service ..."
  sed -i "s|JAVA_HOME=.*|JAVA_HOME=${JAVA_HOME}|" $TALEND_HOME/runtime/bin/talend-runtime-$PROD_VER-service
fi
if [ -f $TALEND_HOME/runtime/etc/talend-runtime-$PROD_VER-wrapper.conf ]; then
  cFile=$TALEND_HOME/runtime/etc/talend-runtime-$PROD_VER-wrapper.conf
  sed -i "s|JAVA_HOME=.*|JAVA_HOME=${JAVA_HOME}|" $cFile
  if [ "$JAVA_11" = "OK" -o "$JAVA_17" = "OK" ]; then
    if [ "`egrep -c -e '^wrapper\.java\.additional\.1=-Djava\.endorsed\.dirs=' $cFile`" = "1" ]; then
      # The script needs to patch this file against Java 11 (remove endorsed dirs)
      for i in 1 2
      do
        sed -i "s|^wrapper\.java\.additional\.$i=|#wrapper.java.additional.$i=|" $cFile
      done

      for i in 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20
      do
        if [ "`grep -c -e \"wrapper\.java\.additional\.$i=\" $cFile`" = "1" ]; then
	  let i2=i-2
	  sed -i "s|wrapper\.java\.additional\.${i}=|wrapper\.java\.additional\.${i2}=|" $cFile
	fi
      done
    fi
  fi
fi

# Updating JobServer service
if [ -f /etc/systemd/system/talend-rjs-${PROD_VER}.service ]; then
  echo "Updating JobServer service..."
  cFile=/etc/systemd/system/talend-rjs-${PROD_VER}.service
  #Extracts the old Java home
  oldJavaHome=`grep "Environment=JAVA_HOME=" $cFile |sed 's|Environment=JAVA_HOME=||'`
  if [ -n "$oldJavaHome" ]; then
    # Try to remove it from Environment PATH element
    if [ -n "`grep 'Environment=PATH=' $cFile | grep -F \"$oldJavaHome\" `" ]; then
      sed -i "s|\(Environment=PATH=.*\):$oldJavaHome|\1|" $cFile
    fi
  fi
  # Now replace JAVA_HOME
  sed -i "s|Environment=JAVA_HOME=.*|Environment=JAVA_HOME=$JAVA_HOME|" $cFile
  # and add JAVA_HOME/bin to the PATH
  sed -i "s|Environment=PATH=\(.*\)|Environment=PATH=$JAVA_HOME/bin:\1|" $cFile

  # Update Agent scripts
  for cScript in start_rs.sh stop_rs.sh
  do
    if [ -f $TALEND_HOME/jobserver/agent/$cScript ]; then
      sed -i "s|MY_JAVA=.*|MY_JAVA=$JAVA_HOME/bin/java|" $TALEND_HOME/jobserver/agent/$cScript
    fi
  done
fi

# Update TDQ Portal (only if Java version != 11)
if [ -f /etc/systemd/system/talend-tdqp-${PROD_VER}.service ]; then
  if [ "$JAVA_11" != "OK" -a "$JAVA_17" != "OK" ]; then
    echo "Updating TDQ Portal service..."
    for cDescr in talend-tdqp talend-tdqp-db
    do
      if [ -f /etc/systemd/system/${cDescr}-${PROD_VER}.service ]; then
        sed -i "s|Environment=JAVA_HOME=.*|Environment=JAVA_HOME=${JAVA_HOME}|" /etc/systemd/system/${cDescr}-${PROD_VER}.service
      fi
    done
    if [ -f $TALEND_HOME/utils/srv_tdqp ]; then
      sed -i "s|JAVA_HOME=.*|JAVA_HOME=${JAVA_HOME}|" $TALEND_HOME/utils/srv_tdqp
    fi
  fi
fi

# Reload updated Systemd descriptor files
systemctl daemon-reload

echo
echo "Java version update is complete."
echo

if [ -f /etc/systemd/system/talend-nexus-${PROD_VER}.service -a "$JAVA_11" = "OK" ]; then
  echo "Warning: Talend Nexus service was not updated to use Java 11."
fi

if [ -f /etc/systemd/system/talend-tdqp-${PROD_VER}.service -a "$JAVA_11" = "OK" ]; then
  echo "Warning: Talend DQ Portal service was not updated to use Java 11."
fi

if [ "$JAVA_17" = "OK" ]; then
  if [ -f /etc/systemd/system/talend-mdm-${PROD_VER}.service ]; then
    echo "Warning: Talend MDM service was not updated for Java 17 and it remains on the old Java version."
  fi
  if [ -f /etc/systemd/system/talend-elastic-${PROD_VER}.service ]; then
    echo "Warning: Talend LogServer service was not updated for Java 17 and it remains on the old Java version."
  fi
fi
